#include "search_star.h"
#include <stdio.h>

static class CatalogName
{
	char	*Description;
  public:
	char	Abbreviation[11];
	char	LongName[100];
	bool	HasNoCatNumbers;
	CatalogName();
	const char *GetDescription() const { return	Description; }
	void SetDescription (const char *desc);
	~CatalogName ();
} *sCatNames = NULL;
static int sNumCatNames = 0;

CatalogName::CatalogName(): Description(NULL), HasNoCatNumbers(false)
{
	Abbreviation[0] = '\0';
	LongName[0] = '\0';
}

void CatalogName::SetDescription (const char *desc)
{
	if (Description)
		delete	Description;
	if (!desc)
		Description = NULL;
	else
	{
		Description = new char [strlen (desc) + 1];
		strcpy (Description, desc);
	}
}

CatalogName::~CatalogName()
{
	if (Description)
		delete	Description;
}

const char *CatAbbrev2LongName (const char *abbrev)
{
	for (register i = 0; i < sNumCatNames; ++i)
		if (!strcmp (abbrev, sCatNames[i].Abbreviation))
			if (sCatNames[i].LongName[0])
				return	sCatNames[i].LongName;
			else
				break;
	return	abbrev;
}

void FetchCatalogNames (HSTMT hstmt)
{
	RETCODE	retcode;

	retcode = SQLFreeStmt (hstmt, SQL_CLOSE);
	retcode = SQLExecDirect (hstmt,
		(unsigned char *) "SELECT COUNT(*) FROM CatalogNames", SQL_NTS);
	if (retcode == SQL_SUCCESS)
	{
		SDWORD	cbCount;

		SQLBindCol (hstmt, 1, SQL_C_LONG, &sNumCatNames, 0, &cbCount);
		retcode = SQLFetch(hstmt);
		if (retcode == SQL_SUCCESS  ||  retcode == SQL_SUCCESS_WITH_INFO)
		{
			retcode = SQLFreeStmt (hstmt, SQL_CLOSE);
			retcode = SQLExecDirect (hstmt, (unsigned char *)
				"SELECT Abbreviation, LongName, Description, HasNoCatNumbers "
				"FROM CatalogNames", SQL_NTS);
			if (retcode == SQL_SUCCESS)
			{
				char	abbrevBuf[11], nameBuf[101], descBuf[2048];
				bool	hasNoCatNumbers;
				SDWORD	cbAbbreviation, cbLongName, cbDescription, cbHasNoCatNumbers;

				sCatNames = new CatalogName [sNumCatNames];
				SQLBindCol (hstmt, 1, SQL_C_CHAR, abbrevBuf, 11, &cbAbbreviation);
				SQLBindCol (hstmt, 2, SQL_C_CHAR, nameBuf, 101, &cbLongName);
				SQLBindCol (hstmt, 3, SQL_C_CHAR, descBuf, 2048, &cbDescription);
				SQLBindCol (hstmt, 4, SQL_C_BIT, &hasNoCatNumbers, 0, &cbHasNoCatNumbers);
				for (int i = 0; i < sNumCatNames; ++i)
				{
					retcode = SQLFetch(hstmt);
					if (retcode != SQL_SUCCESS  &&  retcode != SQL_SUCCESS_WITH_INFO)
						break;
					if (cbAbbreviation != SQL_NULL_DATA)
						strcpy (sCatNames[i].Abbreviation, abbrevBuf);
					if (cbLongName != SQL_NULL_DATA)
						strcpy (sCatNames[i].LongName, nameBuf);
					if (cbDescription != SQL_NULL_DATA)
						sCatNames[i].SetDescription (descBuf);
					sCatNames[i].HasNoCatNumbers = hasNoCatNumbers;
				}
			}
		}
	}
}

void DisposeCatalogNames()
{
	if (sCatNames)
	{
		delete [] sCatNames;
		sCatNames = NULL;
	}
}

bool FetchMainSystemName (HSTMT hstmt, long id, char *nameBuf)
{
	char	buffer[512];
	RETCODE	retcode;

	SQLFreeStmt (hstmt, SQL_CLOSE);
	sprintf (buffer, "SELECT Name FROM ProperNames "
		"WHERE INT(OwnerID/100) = %ld AND IsSystemName = Yes", id/100);
	retcode = SQLExecDirect (hstmt, (unsigned char *) buffer, SQL_NTS);
	if (retcode == SQL_SUCCESS)
	{
		SDWORD	cbName;
		char	name[51];

		SQLBindCol(hstmt, 1, SQL_C_CHAR, name, 51, &cbName);
		retcode = SQLFetch(hstmt);
		if (retcode == SQL_SUCCESS  ||  retcode == SQL_SUCCESS_WITH_INFO)
		{
			strcpy (nameBuf, name);
			return	true;
		}

		SQLFreeStmt (hstmt, SQL_CLOSE);
		sprintf (buffer, "SELECT CatAbbrev, Value FROM CatalogNumbers "
			"WHERE INT(OwnerID/100) = %ld AND IsSystemName = Yes", id/100);
		retcode = SQLExecDirect (hstmt, (unsigned char *) buffer, SQL_NTS);
		if (retcode == SQL_SUCCESS)
		{
			char	catAbbrev[11];
			SDWORD	cbCatAbbrev;

			SQLBindCol(hstmt, 1, SQL_C_CHAR, catAbbrev, 11, &cbCatAbbrev);
			SQLBindCol(hstmt, 2, SQL_C_CHAR, name, 51, &cbName);
			retcode = SQLFetch(hstmt);
			if (retcode == SQL_SUCCESS  ||  retcode == SQL_SUCCESS_WITH_INFO)
			{
				strcpy (nameBuf, CatAbbrev2LongName (catAbbrev));
				strcat (nameBuf, " ");
				strcat (nameBuf, name);
				return	true;
			}
		}
	}
	return	false;
}

bool FetchRawSpectrum (HSTMT hstmt, long id, RawSpectrum &spec)
{
	char	buffer[512];
	RETCODE	retcode;

	retcode = SQLFreeStmt (hstmt, SQL_CLOSE);
	sprintf (buffer, "SELECT SpectralClass, LuminosityClass, "
		"SpectrumSource, LastUpdated FROM Spectra "
		"WHERE OwnerID = %ld", id);
	retcode = SQLExecDirect (hstmt, (unsigned char *) buffer, SQL_NTS);
	if (retcode == SQL_SUCCESS)
	{
		SDWORD	cbSpectralClass, cbLuminosityClass;
		SDWORD	cbSpectrumSource, cbLastUpdated;

		SQLBindCol (hstmt, 1, SQL_C_CHAR, spec.SpectralClass, 51, &cbSpectralClass);
		SQLBindCol (hstmt, 2, SQL_C_CHAR, spec.LuminosityClass, 51, &cbLuminosityClass);
		SQLBindCol (hstmt, 3, SQL_C_CHAR, spec.SpectrumSource, 11, &cbSpectrumSource);
		SQLBindCol (hstmt, 4, SQL_C_DATE, &spec.LastUpdated, 0, &cbLastUpdated);
		retcode = SQLFetch (hstmt);
		if (retcode == SQL_SUCCESS  ||  retcode == SQL_SUCCESS_WITH_INFO)
		{
			spec.bSpectralClass = cbSpectralClass != SQL_NULL_DATA;
			spec.bLuminosityClass = cbLuminosityClass != SQL_NULL_DATA;
			spec.bSpectrumSource = cbSpectrumSource != SQL_NULL_DATA;
			return	true;
		}
	}
	return	false;
}
